// Generated by IcedCoffeeScript 108.0.9
var DnsServer, EntryList, async, dns, iced, logger, __iced_k, __iced_k_noop;

iced = require('iced-runtime');
__iced_k = __iced_k_noop = function() {};

logger = require("./logger");

dns = require("native-dns");

async = require("async");

EntryList = require("./entrylist");

DnsServer = (function() {
  function DnsServer(options) {
    this.port = options.port || 53;
    this.gid = options.gid;
    this.uid = options.uid;
    this.server = dns.createServer();
    this.entries = new EntryList;
    this.authority = {
      address: options.upstream || "8.8.8.8",
      port: 53,
      type: "udp"
    };
    this.server.on("listening", (function(_this) {
      return function() {
        return logger.info("dns server listening on", _this.server.address());
      };
    })(this));
    this.server.on("close", (function(_this) {
      return function() {
        return logger.info("dns server closed");
      };
    })(this));
    this.server.on("error", (function(_this) {
      return function(err, buff, req, res) {
        return logger.error(err.stack);
      };
    })(this));
    this.server.on("socketError", (function(_this) {
      return function(err, socket) {
        return logger.error(err);
      };
    })(this));
    this.server.on("request", (function(_this) {
      return function(req, res) {
        return _this.handleRequest(req, res);
      };
    })(this));
  }

  DnsServer.prototype.start = function() {
    this.server.serve(this.port);
    return this.secure();
  };

  DnsServer.prototype.secure = function() {
    if (process.getuid && process.getgid && process.getuid() === 0) {
      if (this.gid) {
        logger.info("setting gid to", this.gid);
        process.setgid(this.gid);
      }
      if (this.uid) {
        logger.info("setting uid to", this.uid);
        return process.setuid(this.uid);
      }
    }
  };

  DnsServer.prototype.addEntry = function(domain, type, address, ttl) {
    if (type == null) {
      type = "A";
    }
    if (address == null) {
      address = "127.0.0.1";
    }
    if (ttl == null) {
      ttl = 1800;
    }
    return this.entries.addEntry(domain, type, address, ttl);
  };

  DnsServer.prototype.addTextEntry = function(data) {
    return this.entries.addTextEntry(data);
  };

  DnsServer.prototype.proxyQuestion = function(question, response, callback) {
    var request;
    request = dns.Request({
      question: question,
      server: this.authority,
      timeout: 1000
    });
    request.on("message", (function(_this) {
      return function(err, msg) {
        var answer, _i, _len, _ref, _results;
        _ref = msg.answer;
        _results = [];
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          answer = _ref[_i];
          _results.push(response.answer.push(answer));
        }
        return _results;
      };
    })(this));
    request.on("end", callback);
    return request.send();
  };

  DnsServer.prototype.handleRequest = function(request, response) {
    var entries, entry, f, question, record, ___iced_passed_deferral, __iced_deferrals, __iced_k, _i, _j, _len, _len1, _ref;
    __iced_k = __iced_k_noop;
    ___iced_passed_deferral = iced.findDeferral(arguments);
    f = [];
    _ref = request.question;
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      question = _ref[_i];
      entries = this.entries.findEntries(question.name);
      if (entries.length) {
        for (_j = 0, _len1 = entries.length; _j < _len1; _j++) {
          entry = entries[_j];
          record = {
            name: question.name,
            type: entry.type,
            address: entry.address,
            ttl: entry.ttl
          };
          logger.debug("resolving locally", record.name, "to", record.address);
          response.answer.push(dns[record.type](record));
        }
      } else {
        logger.debug("proxying", question.name, "to", this.authority.address);
        f.push((function(_this) {
          return function(callback) {
            return _this.proxyQuestion(question, response, callback);
          };
        })(this));
      }
    }
    (function(_this) {
      return (function(__iced_k) {
        __iced_deferrals = new iced.Deferrals(__iced_k, {
          parent: ___iced_passed_deferral,
          filename: "/home/aarevalo/projects/simple-dns-proxy/src/dns-server.iced",
          funcname: "DnsServer.handleRequest"
        });
        async.parallel(f, __iced_deferrals.defer({
          lineno: 100
        }));
        __iced_deferrals._fulfill();
      });
    })(this)((function(_this) {
      return function() {
        return response.send();
      };
    })(this));
  };

  return DnsServer;

})();

module.exports = DnsServer;
